package cs3651;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Ellipse2D;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JPanel;
import javax.comm;

/**
 *
 * @author James
 */
public class WheelPanel extends JPanel implements MouseListener, MouseMotionListener{

    final double D_TO_R = 0.0174532925;
    int button = 0;
    final int WHEEL_RADIUS_OUT = 220;
    final int WHEEL_RADIUS_IN = 180;
    final int NUMBER_OF_RINGS = 10;
    final int SPACING = 10;
    final int PWIDTH = 500;
    final double ANGLE_OFFSET = 190 * D_TO_R;
    ArrayList<ArrayList<Point>> points;

    byte[][][] CHAR_DATA = {
                 {{0,1,1,0},   //A
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,1},
        	  {1,1,1,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{1,1,1,0},   //B
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,0},
        	  {1,1,1,0},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {1,1,1,0}},
                 {{0,1,1,0},   //C
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,0},
                  {1,0,0,0},
        	  {1,0,0,0},
		  {1,0,0,0},
		  {1,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{1,1,0,0},    //D
                  {1,1,1,0},
                  {1,0,1,0},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,1,0},
		  {1,1,1,0},
		  {1,1,0,0}},
                 {{1,1,1,1},    //E
                  {1,1,1,1},
                  {1,0,0,0},
                  {1,0,0,0},
                  {1,1,1,0},
        	  {1,1,1,0},
		  {1,0,0,0},
		  {1,0,0,0},
		  {1,1,1,1},
		  {1,1,1,1}},    //F
                 {{1,1,1,1},
                  {1,1,1,1},
                  {1,0,0,0},
                  {1,0,0,0},
                  {1,1,1,0},
        	  {1,1,1,0},
		  {1,0,0,0},
		  {1,0,0,0},
		  {1,0,0,0},
		  {1,0,0,0}},
                 {{0,1,1,0},    //G
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,0},
                  {1,0,0,0},
        	  {1,0,1,1},
		  {1,0,1,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{1,0,0,1},    //H
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,1},
        	  {1,1,1,0},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{1,1,1,1},   //I
                  {1,1,1,1},
                  {0,1,1,0},
                  {0,1,1,0},
                  {0,1,1,0},
        	  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {1,1,1,1},
		  {1,1,1,1}},   //J
                 {{1,1,1,1},
                  {1,1,1,1},
                  {0,0,1,0},
                  {0,0,1,0},
                  {0,0,1,0},
        	  {0,0,1,0},
		  {1,0,1,0},
		  {1,0,1,0},
		  {1,0,1,0},
		  {0,1,0,0}},
                  {{1,0,0,1},    //K
                  {1,0,0,1},
                  {1,0,1,0},
                  {1,0,1,0},
                  {1,1,1,0},
        	  {1,1,0,0},
		  {1,1,1,0},
		  {1,0,1,0},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{1,1,0,0},   //L
                  {1,1,0,0},
                  {1,1,0,0},
                  {1,1,0,0},
                  {1,1,0,0},
        	  {1,1,0,0},
		  {1,1,0,0},
		  {1,1,0,0},
		  {1,1,1,1},
		  {1,1,1,1}},
                 {{0,0,0,1},   //M
                  {1,1,1,1},
                  {1,1,1,1},
                  {1,1,1,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{1,0,0,1},    //N
                  {1,0,0,1},
                  {1,1,0,1},
                  {1,1,0,1},
                  {1,1,0,1},
        	  {1,0,1,1},
		  {1,0,1,1},
		  {1,0,1,1},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{0,1,1,0},   //O
                  {0,1,1,0},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{1,1,1,0},  //P
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,1},
        	  {1,1,1,0},
		  {1,0,0,0},
		  {1,0,0,0},
		  {1,0,0,0},
		  {1,0,0,0}},
                 {{0,1,1,0},  //Q
                  {0,1,1,0},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,1,1},
		  {1,0,1,0},
		  {0,1,1,1},
		  {0,1,0,1}},
                 {{1,1,1,0},  //R
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,1},
        	  {1,1,1,0},
		  {1,1,0,0},
		  {1,1,1,0},
		  {1,0,1,1},
		  {1,0,0,1}},
                 {{0,1,1,0},  //S
                  {0,1,1,1},
                  {1,0,0,1},
                  {1,0,0,0},
                  {1,1,1,0},
        	  {0,1,1,1},
		  {0,0,0,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},  //T
                 {{1,1,1,1},
                  {1,1,1,1},
                  {0,1,1,0},
                  {0,1,1,0},
                  {0,1,1,0},
        	  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{1,0,0,1},    //U
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{1,0,0,1},   //V
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{1,0,0,1},   //W
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,1,1,1},
		  {1,1,1,1},
		  {1,1,1,1},
		  {1,0,0,1}},
                 {{1,0,0,1},    //X
                  {1,0,0,1},
                  {1,0,0,1},
                  {0,1,1,0},
                  {0,1,1,0},
        	  {0,1,1,0},
		  {0,1,1,0},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1}},
                 {{1,0,0,1},    //Y
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,1,1,1},
        	  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{1,1,1,1},    //Z
                  {1,1,1,1},
                  {0,0,0,1},
                  {0,0,1,1},
                  {0,0,1,0},
        	  {0,1,0,0},
		  {1,1,0,0},
		  {1,0,0,0},
		  {1,1,1,1},
		  {1,1,1,1}},
                 {{0,1,1,0},   //O
                  {0,1,1,0},
                  {1,0,0,1},
                  {1,0,0,1},
                  {1,0,0,1},
        	  {1,0,0,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{0,1,1,0},    //1
                  {1,1,1,0},
                  {1,1,1,0},
                  {0,1,1,0},
                  {0,1,1,0},
        	  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {1,1,1,1},
		  {1,1,1,1}},
                 {{0,1,1,0},    //2
                  {0,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {0,0,1,0},
        	  {0,0,1,0},
		  {0,1,0,0},
		  {0,1,0,0},
		  {1,1,1,1},
		  {1,1,1,1}},
                 {{0,1,1,0},    //3
                  {1,0,1,1},
                  {1,0,0,1},
                  {0,0,0,1},
                  {0,1,1,0},
        	  {0,0,0,1},
		  {0,0,0,1},
		  {1,0,0,1},
		  {1,0,1,1},
		  {0,1,1,0}},
                 {{1,0,1,0},    //4
                  {1,0,1,0},
                  {1,0,1,0},
                  {1,1,1,1},
                  {1,1,1,1},
        	  {0,0,1,0},
		  {0,0,1,0},
		  {0,0,1,0},
		  {0,0,1,0},
		  {0,0,1,0}},
                 {{1,1,1,1},    //5
                  {1,1,1,1},
                  {1,0,0,0},
                  {1,0,0,0},
                  {1,1,1,0},
        	  {1,1,1,1},
		  {0,0,0,1},
		  {0,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{0,1,1,0},    //6
                  {0,1,0,1},
                  {1,0,0,0},
                  {1,0,0,0},
                  {1,1,1,0},
        	  {1,1,1,1},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{1,1,1,1},    //7
                  {1,1,1,1},
                  {0,0,0,1},
                  {0,0,1,1},
                  {0,0,1,1},
        	  {0,1,1,1},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0},
		  {0,1,1,0}},
                 {{0,1,1,0},    //8
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {0,1,1,0},
        	  {0,1,1,0},
		  {1,0,0,1},
		  {1,0,0,1},
		  {1,1,1,1},
		  {0,1,1,0}},
                 {{0,1,1,0},    //9
                  {1,1,1,1},
                  {1,0,0,1},
                  {1,0,0,1},
                  {0,1,1,1},
        	  {0,1,1,1},
		  {0,0,1,1},
		  {0,0,1,1},
		  {0,0,1,1},
		  {0,0,1,1}},
                 
};


    public WheelPanel() {
        points = new ArrayList();
        setPreferredSize(new Dimension(PWIDTH,PWIDTH));
        addMouseListener(this);
        addMouseMotionListener(this);
        generatePoints();
    }

    void generatePoints(){
        double distance = ((double) (WHEEL_RADIUS_OUT - WHEEL_RADIUS_IN)) / NUMBER_OF_RINGS;
        for(int i = 1; i <= NUMBER_OF_RINGS; i++){
            ArrayList<Point> temp = new ArrayList();
            double radius = WHEEL_RADIUS_IN + distance * i;
            double circumference = 2 * Math.PI * radius;
            //double number = Math.floor(circumference / SPACING);
            double number = 123;
            double theta = 2 * Math.PI / number;
            for(int j = 0;  j < number; j++){
                double x = (Math.cos(-theta * j) * radius + PWIDTH / 2);
                double y = (Math.sin(-theta * j) * radius + PWIDTH / 2);
                temp.add(new Point(x,y,theta,false));
            }
            points.add(temp);
        }
    }

    @Override
    public void paint(Graphics g){
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(Color.white);
        g2.fillRect(0, 0, PWIDTH, PWIDTH);
        g2.setColor(Color.blue);
        for(ArrayList<Point> a: points){
            for(Point p: a){
                if(p.on){
                    g2.fill(new Ellipse2D.Double(p.x-3,p.y-3,6,6));
                }else{
                    g2.draw(new Ellipse2D.Double(p.x-2,p.y-2,4,4));
                }
            }
        }
    }

    public Point closestPoint(int x, int y){
        double x2 = Math.abs(x - 250);
        double y2 = Math.abs(y - 250);
        double distance = Math.sqrt(Math.pow(x2,2) + Math.pow(y2, 2));
        double spacing = ((double)(WHEEL_RADIUS_OUT - WHEEL_RADIUS_IN)) / NUMBER_OF_RINGS;
        int ring;
        if((distance - WHEEL_RADIUS_IN) % spacing > spacing / 2){
            ring = (int)Math.ceil((distance - WHEEL_RADIUS_IN) / spacing);
        }else{
            ring = (int)Math.floor((distance - WHEEL_RADIUS_IN)/ spacing);
        }
        if(ring > NUMBER_OF_RINGS || ring <= 0)return null;
        double theta = Math.asin(y2 / distance);
        System.out.println(theta);
        if(x < 250 && y < 250) theta = Math.PI - theta;
        if(x < 250 && y >= 250) theta = Math.PI + theta;
        if(x >= 250 && y >= 250) theta = 2 * Math.PI - theta;



        ArrayList<Point> pointList = points.get(ring - 1);
        double step = Math.PI * 2 / pointList.size();
        int index;
        if(theta % step > step / 2){
            index = (int)Math.ceil(theta / step);
        }else{
            index = (int)Math.floor(theta / step);
        }
        if(index == pointList.size()) index = 0;
        return pointList.get(index);
    }

    public void mouseClicked(MouseEvent e) {
        
    }

    public void mousePressed(MouseEvent e) {
        Point p = closestPoint(e.getX(), e.getY());
        if(p == null) return;
        if(p.on){
            p.on = false;
            button = 2;
        }else {
            p.on = true;
            button = 1;
        }
        repaint();
    }

    public void mouseReleased(MouseEvent e) {
            button = 0;
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {

    }

    void clear() {
        for(ArrayList<Point> a: points){
            for(Point p: a){
                p.on = false;
            }
        }
    }

    void save() {
        FileWriter outputStream = null;

        try{
            outputStream = new FileWriter("cs3651.dat");
            for(ArrayList<Point> a: points){
                for(Point p: a){
                    if(p.on) outputStream.write("1");
                    else outputStream.write("0");
                }
            }
            outputStream.close();
        }catch(IOException ex){
            System.exit(1);
        }


    }

    void load() {


        FileReader inputStream = null;
        try {
            inputStream = new FileReader("cs3651.dat");
            int c;
            for(ArrayList<Point> a: points){
                for(Point p: a){
                    c = inputStream.read();
                    if(c == -1){
                        System.exit(1);
                    }else if(c == 48){
                        p.on = false;
                    }else if(c == 49){
                        p.on = true;
                    }
                }
            }
            inputStream.close();
        }catch(IOException ex){
            System.exit(1);
        }
        repaint();
    }
    
    void upload() {
    	
    	
    }

    void export() {
        FileWriter outputStream = null;
        try{
            outputStream = new FileWriter("cs3651.pde");
            outputStream.write("int lastRead = HIGH;\n");
            outputStream.write("long lastUpdate = 0;\n");
            outputStream.write("long lastDeadFlash = 0;\n");
            outputStream.write("long lastMillis = 0;\n");
            outputStream.write("boolean dead = false;\n");
            outputStream.write("double angularSpeed = 0;\n");
            outputStream.write("int currentRead;\n");
            outputStream.write("long currentMillis;\n\n");
            for(int j = 0; j < points.size(); j++){
                ArrayList<Point> a = points.get(j);
                outputStream.write("boolean ring" + (j+1) + "[" + a.size() + "] = {");
                if(a.get(0).on) outputStream.write("true");
                else outputStream.write("false");
                for(int i = 1; i < a.size(); i++){
                    outputStream.write(",");
                    if(a.get(i).on) outputStream.write("true");
                    else outputStream.write("false");
                }outputStream.write("};\n");
            }
            outputStream.write("\n\nboolean* rings[10] = {ring1, ring2, ring3, ring4, ring5, ring6, ring7, ring8, ring9, ring10};\n");
            outputStream.write("int sizes[10] = {");
            outputStream.write("" + points.get(0).size());
            for(int i = 1; i <=9; i++){
                outputStream.write("," + points.get(i).size());
            }
            outputStream.write("};\n\n");

            outputStream.write("double currentPosition;\n");
            outputStream.write("int updatesPerRotation;\n");
            outputStream.write("double inchesTravelled = 0;\n");
            outputStream.write("double currentSpeed = 0;\n");
            outputStream.write("double averageSpeed = 0;\n");
            outputStream.write("double maxSpeed = 0;\n");
            outputStream.write("double numberOfRotations = 0;\n\n");
            outputStream.write("void setup() { \n");
            outputStream.write("\tSerial.begin(57600);\n");
            outputStream.write("\tfor (int i = 2; i < 12; i++)\n");
            outputStream.write("\t\tpinMode(i, OUTPUT);\n");
            outputStream.write("\tpinMode(12, INPUT);\n");
            outputStream.write("\tpinMode(13, OUTPUT);\n");
            outputStream.write("\tlastMillis = millis();\n");
            outputStream.write("\tlastUpdate = millis();\n");
            outputStream.write("}\n\n");
            outputStream.write("void loop() {\n");
            outputStream.write("\tcheckInput();\n");
            outputStream.write("\tcheckSpin();\n");
            outputStream.write("\tif (!dead) {\n");
            outputStream.write("\t\tresetAll();\n");
            outputStream.write("\t\tupdateSpeed();\n");
            outputStream.write("\t\tdeterminePosition();\n");
            outputStream.write("\t\tlightPixels();\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void checkInput() {\n");
            outputStream.write("\tif (Serial.available()) {\n");
            outputStream.write("\t\tbyte command = Serial.read();\n");
            outputStream.write("\t\tif (command == 'c') {\n");
            outputStream.write("\t\t\tif (dead) Serial.println(\"Dead.\");\n");
            outputStream.write("\t\t\tSerial.println(angularSpeed);\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t\telse if (command == 't') {\n");
            outputStream.write("\t\t\tSerial.println(\"Your Trip Information (since last reset):\");\n");
            outputStream.write("\t\t\tSerial.print(\"Distance Travelled: \");\n");
            outputStream.write("\t\t\tSerial.print(inchesTravelled);\n");
            outputStream.write("\t\t\tSerial.print(\" inches, or \");\n");
            outputStream.write("\t\t\tSerial.print(inchesTravelled/(12.0 * 5280.0));\n");
            outputStream.write("\t\t\tSerial.println(\" miles.\");\n");
            outputStream.write("\t\t\tSerial.print(\"Average speed: \");\n");
            outputStream.write("\t\t\tSerial.print(averageSpeed);\n");
            outputStream.write("\t\t\tSerial.print(\" inches per second, or \");\n");
            outputStream.write("\t\t\tSerial.print(averageSpeed/(12.0 * 5280.0));\n");
            outputStream.write("\t\t\tSerial.println(\" miles per hour.\");\n");
            outputStream.write("\t\t\tSerial.print(\"Max speed: \");\n");
            outputStream.write("\t\t\tSerial.print(maxSpeed);\n");
            outputStream.write("\t\t\tSerial.print(\" inches per second, or \");\n");
            outputStream.write("\t\t\tSerial.print(maxSpeed/(12.0 * 5280.0));\n");
            outputStream.write("\t\t\tSerial.println(\" miles per hour.\");\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void checkSpin() {\n");
            outputStream.write("\tcurrentRead = digitalRead(12);\n");
            outputStream.write("\tcurrentMillis = millis();\n");
            outputStream.write("\tif (currentRead == lastRead) {\n");
            outputStream.write("\t\tif (currentMillis - lastMillis > 1000) {\n");
            outputStream.write("\t\t\tdead = true;\n");
            outputStream.write("\t\t\tlastMillis = currentMillis;\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse{\n");
            outputStream.write("\t\tdead = false;\n");
            outputStream.write("\t\tlastMillis = currentMillis;\n");
            outputStream.write("\t}\n");
            outputStream.write("\tif (dead) {\n");
            outputStream.write("\t\tsetDead();\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void resetAll() {\n");
            outputStream.write("\tfor (int i = 2; i < 12; i++) {\n");
            outputStream.write("\t\tdigitalWrite(i, LOW);\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void updateSpeed() {\n");
            outputStream.write("\tif (currentRead == LOW && currentRead != lastRead) {\n");
            outputStream.write("\t\tcurrentPosition = 0;\n");
            outputStream.write("\t\tlong time = currentMillis - lastUpdate;\n");
            outputStream.write("\t\tlastUpdate = currentMillis;\n");
            outputStream.write("\t\tangularSpeed = (100.0) / (time / 1000.0);\n");
            outputStream.write("\t\tupdatesPerRotation = 0;\n");
            outputStream.write("\t\tinchesTravelled += 3.14159265 *  27;   \n");
            outputStream.write("\t\tcurrentSpeed = angularSpeed * 0.8482300165;\n");
            outputStream.write("\t\tnumberOfRotations++;\n");
            outputStream.write("\t\taverageSpeed = (averageSpeed + currentSpeed) / 2.0;\n");
            outputStream.write("\t\tmaxSpeed = (currentSpeed > maxSpeed) ? currentSpeed : maxSpeed;\n");
            outputStream.write("\t}\n");
            outputStream.write("\tlastRead = currentRead;\n");
            outputStream.write("}\n\n");
            outputStream.write("void setDead() {\n");
            outputStream.write("\tif (currentMillis - lastDeadFlash < 50) {\n");
            outputStream.write("\t\tdigitalWrite(2, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 100) {\n");
            outputStream.write("\t\tdigitalWrite(2, LOW);\n");
            outputStream.write("\t\tdigitalWrite(3, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 150) {\n");
            outputStream.write("\t\tdigitalWrite(3, LOW);\n");
            outputStream.write("\t\tdigitalWrite(4, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 200) {\n");
            outputStream.write("\t\tdigitalWrite(4, LOW);\n");
            outputStream.write("\t\tdigitalWrite(5, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 250) {\n");
            outputStream.write("\t\tdigitalWrite(5, LOW);\n");
            outputStream.write("\t\tdigitalWrite(6, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 300) {\n");
            outputStream.write("\t\tdigitalWrite(6, LOW);\n");
            outputStream.write("\t\tdigitalWrite(7, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 350) {\n");
            outputStream.write("\t\tdigitalWrite(7, LOW);\n");
            outputStream.write("\t\tdigitalWrite(8, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 400) {\n");
            outputStream.write("\t\tdigitalWrite(8, LOW);\n");
            outputStream.write("\t\tdigitalWrite(9, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 450) {\n");
            outputStream.write("\t\tdigitalWrite(9, LOW);\n");
            outputStream.write("\t\tdigitalWrite(10, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 500) {\n");
            outputStream.write("\t\tdigitalWrite(10, LOW);\n");
            outputStream.write("\t\tdigitalWrite(11, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 550) {\n");
            outputStream.write("\t\tdigitalWrite(11, LOW);\n");
            outputStream.write("\t\tdigitalWrite(10, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 600) {\n");
            outputStream.write("\t\tdigitalWrite(10, LOW);\n");
            outputStream.write("\t\tdigitalWrite(9, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 650) {\n");
            outputStream.write("\t\tdigitalWrite(9, LOW);\n");
            outputStream.write("\t\tdigitalWrite(8, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 700) {\n");
            outputStream.write("\t\tdigitalWrite(8, LOW);\n");
            outputStream.write("\t\tdigitalWrite(7, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 750) {\n");
            outputStream.write("\t\tdigitalWrite(7, LOW);\n");
            outputStream.write("\t\tdigitalWrite(6, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 800) {\n");
            outputStream.write("\t\tdigitalWrite(6, LOW);\n");
            outputStream.write("\t\tdigitalWrite(5, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 850) {\n");
            outputStream.write("\t\tdigitalWrite(5, LOW);\n");
            outputStream.write("\t\tdigitalWrite(4, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 900) {\n");
            outputStream.write("\t\tdigitalWrite(4, LOW);\n");
            outputStream.write("\t\tdigitalWrite(3, HIGH);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (currentMillis - lastDeadFlash < 950) {\n");
            outputStream.write("\t\tdigitalWrite(3, LOW);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse {\n");
            outputStream.write("\t\tlastDeadFlash = currentMillis;\n");
            outputStream.write("\t\tcurrentPosition = 0;\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void setPixel(int x, int y) {\n");
            outputStream.write("\tif (x < 0 || x > 9) {\n");
            outputStream.write("\t\tSerial.print(\"Invalid x: \");\n");
            outputStream.write("\t\tSerial.println(x);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse if (y >= sizes[x]) {\n");
            outputStream.write("\t\tSerial.print(\"Invalid y: \");\n");
            outputStream.write("\t\tSerial.println(y);\n");
            outputStream.write("\t}\n");
            outputStream.write("\telse {\n");
            outputStream.write("\t\trings[x][y] = true;\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            outputStream.write("void determinePosition() {\n");
            outputStream.write("\tlong changeInTime = currentMillis - lastUpdate;\n");
            outputStream.write("\tdouble changeInPosition = (angularSpeed) * (changeInTime/1000.0);\n");
            outputStream.write("\tcurrentPosition = changeInPosition;\n");
            outputStream.write("}\n\n");
            outputStream.write("void lightPixels() {\n");
            outputStream.write("\tfor(int i = 0; i < 10 && currentPosition <= 100; i++){\n");
            outputStream.write("\t\tboolean* pointer = rings[i];\n");
            outputStream.write("\t\tboolean b = pointer[(int)(sizes[i] * (currentPosition/100.0))];\n");
            outputStream.write("\t\tif(b){\n");
            outputStream.write("\t\t\tdigitalWrite(i + 2, HIGH);\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t}\n");
            outputStream.write("\tupdatesPerRotation++;\n");
            outputStream.write("}\n");
            outputStream.close();
        }catch(IOException ex){
            System.exit(1);
        }
    }

    /*
    void export() {
        FileWriter outputStream = null;
        try{
            outputStream = new FileWriter("cs3651.pde");
            outputStream.write("volatile unsigned long firstTime;\n");
            outputStream.write("volatile unsigned long secondTime;\n");
            outputStream.write("volatile float rotationTime;\n\n");
            for(int j = 0; j < points.size(); j++){
                ArrayList<Point> a = points.get(j);
                outputStream.write("boolean ring" + (j+1) + "[" + a.size() + "] = {");
                if(a.get(0).on) outputStream.write("true");
                else outputStream.write("false");
                for(int i = 1; i < a.size(); i++){
                    outputStream.write(",");
                    if(a.get(i).on) outputStream.write("true");
                    else outputStream.write("false");
                }outputStream.write("};\n");
            }
            outputStream.write("\n\nboolean* rings[10] = {ring1, ring2, ring3, ring4, ring5, ring6, ring7, ring8, ring9, ring10};\n");
            outputStream.write("int sizes[10] = {");
            outputStream.write("" + points.get(0).size());
            for(int i = 1; i <=9; i++){
                outputStream.write("," + points.get(i).size());
            }
            outputStream.write("};\n");
            outputStream.write("int last;\n\n");
            outputStream.write("void setup(){\n");
            outputStream.write("\tfirstTime = 0;\n");
            outputStream.write("\tsecondTime = 0;\n");
            outputStream.write("\trotationTime = 1000000;\n");
            outputStream.write("\tfor(int i = 2; i<=11; i++){\n");
            outputStream.write("\t\tpinMode(i,OUTPUT);\n");
            outputStream.write("\t}\n");
            outputStream.write("\tpinMode(12,INPUT);\n");
            outputStream.write("}\n\n");
            outputStream.write("void loop(){\n");
            outputStream.write("\tint current = digitalRead(12);\n");
            outputStream.write("\tif(current != last){\n");
            outputStream.write("\t\tlast = current;\n");
            outputStream.write("\t\tif(current == HIGH){\n");
            outputStream.write("\t\t\tfirstTime = micros();\n");
            outputStream.write("\t\t}else{\n");
            outputStream.write("\t\t\tsecondTime = micros();\n");
            outputStream.write("\t\t\trotationTime = (secondTime - firstTime * 4.8);\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t}\n");
            outputStream.write("\tfloat percentage = ((micros() - secondTime) / rotationTime);\n");
            outputStream.write("\tpercentage = percentage - (int) percentage; \n");
            outputStream.write("\tfor(int i = 0; i < 10; i++){\n");
            outputStream.write("\t\tbyte* pointer = rings[i];\n");
            outputStream.write("\t\tbyte b = pointer[(int)(sizes[i] * percentage)];\n");
            outputStream.write("\t\tif(b == 1){\n");
            outputStream.write("\t\t\tdigitalWrite(i + 2, HIGH);\n");
            outputStream.write("\t\t}else{\n");
            outputStream.write("\t\t\tdigitalWrite(i + 2, LOW);\n");
            outputStream.write("\t\t}\n");
            outputStream.write("\t}\n");
            outputStream.write("}\n\n");
            //outputStream.write("void hall(){\n\tif(digitalRead(12) == LOW){\n\t\tfirstTime = micros();\n\t}else{\n\t\tsecondTime = micros();\n\t\trotationTime = (secondTime - firstTime) * 4.8;\n\t}\n}");
            /*for(ArrayList<Point> a: points){
                for(Point p: a){
                    if(p.on) outputStream.write("1");
                    else outputStream.write("0");
                }
            }
            outputStream.close();
        }catch(IOException ex){
            System.exit(1);
        }
    }*/

    void textTop(String text) {
        if(text.length() > 10){
            text = text.substring(0, 10);
        }
        int start = 10 - text.length();
        start /=2;
        start = 50 - start * 5;
        for(int i = 0; i < text.length(); i++){
            drawCharacter1(start, text.charAt(i));
            start -= 5;
        }
    }

    void textBottom(String text){
        if(text.length() > 10){
            text = text.substring(0, 10);
        }
        int start = 10 - text.length();
        start /=2;
        start = 65 + start * 5;
        for(int i = 0; i < text.length(); i++){
            drawCharacter2(start, text.charAt(i));
            start += 5;
        }
    }


    void drawCharacter1(int start, char c){
        if(c == 32) return;
        if(c >= 65 && c <= 90){
            c -= 65;
        }else if(c >= 97 && c <= 122){
            c -= 97;
        }else if(c >= 48 && c <= 57){
            c -= 22;
        }
        if(c < 0 || c > 35) return;
        Point p;
        for(int i = 0; i < 10; i++){
            for(int j = 0; j < 4; j++){
                p = points.get(9 - i).get(start - j);
                if(CHAR_DATA[c][i][j] == 1){
                    p.on = true;
                }else {
                    p.on = false;
                }
            }
        }
    }

    void drawCharacter2(int start, char c){
        if(c >= 65 && c <= 90){
            c -= 65;
        }else if(c >= 97 && c <= 122){
            c -= 97;
        }else if(c >= 48 && c <= 57){
            c -= 22;
        }
        if(c < 0 || c > 35) return;
        Point p;
        for(int i = 0; i < 10; i++){
            for(int j = 0; j < 4; j++){
                p = points.get(i).get(start + j);
                if(CHAR_DATA[c][i][j] == 1){
                    p.on = true;
                }else {
                    p.on = false;
                }
            }
        }
    }

    public void mouseDragged(MouseEvent e) {
        Point p = closestPoint(e.getX(), e.getY());
        if(p == null) return;
        if(button == 2){
            p.on = false;
        }else if(button == 1){
            p.on = true;
        }
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    
    }
}



